/**
 ** worldModelPlannerMainCodels.cc
 **
 ** Codels called by execution task worldModelPlannerMain
 **
 ** Author: Alireza NAKHAEI
 ** Date: March 14, 2009
 **
 **/

#include <portLib.h>

#include "server/worldModelPlannerHeader.h"


#include "hppWorldModelPlanner/hppWorldModelPlanner.h"

#include "server/worldModelGrid3DGPosterLib.h"

#include <err.h>
#include <iostream>
#include <portLib.h>
#include "KineoWorks2/kwsConfig.h"

#include "KineoModel/kppLicense.h"
#include "KineoModel/kppJointComponent.h"
#include "genom-openhrp/genom-hrp2.h"
#include "netshring.h"
#include "hppCorbaServer/hppciOpenHrp.h"
#include "hrp2_14/hrp2_14.h"
#include "hrp2_14/configSwap.h"

#include "worldModelPlanner-config.h"

#define UNKNOWN_MIN 0.3
#define UNKNOWN_MAX 3

#if DEBUG==2
#define ODEBUG2(x) std::cout << "worldModelPlanner:" << x << std::endl
#define ODEBUG1(x) std::cerr << "worldModelPlanner:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "worldModelPlanner:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif


/* --- local data ---------------------------------------------------- */

static POSTER_ID worldModelGrid3DPosterId;		/* current 3d model poster */
static map3D *map3DModel;		/* current poster addr */

static netshring_id_t shtarget;		/**< shared trajectory buffer */
static ChppWorldModelPlanner *planner;	/**< ChppWorldModelPlanner */

/*------------------------------------------------------------------------
 *
 * worldModelPlannerMainInit  --  Initialization codel (fIDS, ...)
 *
 * Description:
 *
 * Returns:    OK or ERROR
 */

STATUS
worldModelPlannerMainInit ( int *report )
{
	SDI_F->initialized = 0;
	/* create shared trajectory buffer */
	shtarget = netshring_create ( WORLDMODEL_PLANNER_TARGET_CONFIG_NAME );

	if ( !shtarget )
	{
		warnx ( "cannot create shared buffer" );
		return ERROR;
	}
	return OK;
}
/*------------------------------------------------------------------------
 *
 * worldModelPlannerMainEnd  --  Termination codel
 *
 * Description:
 *
 * Returns:    OK or ERROR
 */

STATUS
worldModelPlannerMainEnd ( void )
{
	/* delete shared trajectory buffer */
	if ( shtarget )
	{
		netshring_delete ( shtarget );
		shtarget = NULL;
	}
	return OK;
}
/*------------------------------------------------------------------------
 * Init
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_NAMESERVER_NOT_FOUND
 *              S_worldModelPlanner_MODELLOADERJRL_NOT_FOUND
 *              S_worldModelPlanner_OPENHRP_NOT_FOUND
 *              S_worldModelPlanner_KINEOWORKS_LICENSE_NOT_FOUND
 *              S_worldModelPlanner_CORBA_NOT_FOUND
 *              S_worldModelPlanner_HPP_PLANNER_NOT_FOUND
 *              S_worldModelPlanner_COULD_NOT_BUILD_ROBOT
 *              S_worldModelPlanner_ALREADY_INITIALIZED
 *              S_worldModelPlanner_COULD_NOT_ADD_ROBOT
 *              S_worldModelPlanner_COULD_NOT_ADD_OBSTACLE
 *              S_worldModelPlanner_COULD_NOT_INIT_ENVIRONMENT
 */

/* worldModelPlannerInitMain  -  codel EXEC of Init
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerInitMain (int *report )
{
// ----------- 0. init KineoWorks  -------------------------------
	ODEBUG2 ( "Initializing CkppLicense......." );

	if ( !CkppLicense::initialize() )
	{
		ODEBUG1 ( "License for KPP SDK NOT FOUND" );
		*report = S_worldModelPlanner_KINEOWORKS_LICENSE_NOT_FOUND;
		return ETHER;
	}
	ODEBUG2 ( "CkppLicense was found" );
	std::string pathToHrp2 ( PATH_TO_HRP2 );

	planner = new ChppWorldModelPlanner(0.2, 0.2, 0.2, 0.04, pathToHrp2.c_str());
	if ( planner == NULL )
	{
		ODEBUG1 ( "COULD NOT CREATE a planner" );
		*report = S_worldModelPlanner_HPP_PLANNER_NOT_FOUND;
		return ETHER;
	}
	ODEBUG2 ( "planner was created" );
	
	double inVoxelSize;
	double xStart, yStart, zStart, xEnd, yEnd, zEnd;
	
	cerr << "DEBUG 4 - Initializing the POster...." << endl;
// initializing the WorldModelGrid3DG

	fprintf ( stderr, "looking for poster ", WORLDMODELGRID3DG_GRIDTHREEDMAP_POSTER_NAME, "\n" );
	if ( posterFind ( WORLDMODELGRID3DG_GRIDTHREEDMAP_POSTER_NAME, &worldModelGrid3DPosterId ) != OK )
	{
		ODEBUG1 ( "ERROR in Initializing The Poster of  worldModelGrid3DPosterId" );

		map3DModel = NULL;
		fprintf ( stderr,
		          "poster ", "worldModelGrid3DGGriDThreeDMap", " not found, using 2D only.\n" );
	}
	else
	{
		ODEBUG2 ( "Initializing the poster was done" );
		map3DModel = ( map3D * ) posterAddr ( worldModelGrid3DPosterId );
		ODEBUG2 ( "DEBUG 5 - Initializing the poster was done" );
	}
	if ( posterTake ( worldModelGrid3DPosterId, POSTER_READ ) != OK )
	{
		*report = errnoGet();
		ODEBUG1 ( " Error in Poster take." );
		return FAIL;
	}

	ODEBUG2 ( " Poster of worldModelGrid3DPoster was taken" );

	inVoxelSize = map3DModel->map3DResolution;
	ODEBUG2 ( "Resolution = " << inVoxelSize );

	xStart = map3DModel->map3DBoxArea.xStart;
	yStart = map3DModel->map3DBoxArea.yStart;
	zStart = map3DModel->map3DBoxArea.zStart;
	xEnd = map3DModel->map3DBoxArea.xEnd;
	yEnd = map3DModel->map3DBoxArea.yEnd;
	zEnd = map3DModel->map3DBoxArea.zEnd;

	ODEBUG2 ( " Start of The Map = " << xStart <<" , "<< yStart <<" , "<< zStart );
	ODEBUG2 ( " End of The Map = " << xEnd <<" , "<< yEnd <<" , "<< zEnd );

	posterGive ( worldModelGrid3DPosterId );

	if ( planner->setResolution ( inVoxelSize ) ==KD_ERROR )
	{
		ODEBUG1 ( "ERROR :  SetResolution of the planner." );
		return FAIL ;

	}
	ODEBUG2 ( "Resolution was set" );

	if ( planner->allocateMap ( xStart, yStart, zStart, xEnd, yEnd, zEnd ) ==KD_ERROR )
	{
		ODEBUG1 ( "ERROR :  allocateMap of the planner." );
		return FAIL ;

	}
	ODEBUG2 ( "END POSTER - INIT." );

	// ---------- 2. Load Robot Model from ModelLoader ------------------
	ChppciOpenHrpClient openHrpCorbaClient ( planner );
	if ( openHrpCorbaClient.loadHrp2Model(0.02, pathToHrp2.c_str()) != KD_OK )
	{
		*report = S_worldModelPlanner_COULD_NOT_BUILD_ROBOT;
		return FAIL;
	}
	// ---------- 3. Initialize the planner with the devices -------
	// Create bounding box robot and place it at beginning of problem vector.
	if ( planner->initializeProblem() ==KD_ERROR )
	{
		ODEBUG1 ( "worldModelPlanner :  INITIALIZING Problem failed" );
		return FAIL;
	}

		// Set HRP2 in half-sitting position.
	CkppDeviceComponentShPtr hrp2 = planner->robotIthProblem ( 1 );
	CkwsConfigShPtr halfSittingConfig = CkwsConfig::create ( hrp2 ) ;
	double dInitPos[46] = HALFSITTINGPOSITION_RAD_KINEO ;
	std::vector<double>  halfSittingVector ( dInitPos  , dInitPos + sizeof ( dInitPos ) / sizeof ( double ) );
	halfSittingConfig->setDofValues ( halfSittingVector ) ;
	hrp2->setCurrentConfig ( *halfSittingConfig );

	// ---------- 4. Initialize the Environmet -------
	SDI_F->initialized = 1;
	ODEBUG2 ( "Init finish successfully" );
	return ETHER;
}

/*------------------------------------------------------------------------
 * LoadObstacle
 *
 * Description: 
 *
 * Reports:      OK
 *              S_worldModelPlanner_LOAD_OBSTACLE_FAILURE
 */

/* worldModelPlannerLoadObstacle  -  codel EXEC of LoadObstacle
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerLoadObstacle(int *report)
{
	std::vector<char> outRangeMap;
	if ( posterTake ( worldModelGrid3DPosterId, POSTER_READ ) != OK )
	{
		*report = errnoGet();
		return FAIL;
	}
	int inNbOfVoxel = map3DModel->nbOfGridX * map3DModel->nbOfGridY * map3DModel->nbOfGridZ;
	outRangeMap.resize ( inNbOfVoxel );
	for ( int i=0 ; i<inNbOfVoxel ;i++ )
	{
		outRangeMap[i] = map3DModel->gridvalue[i];
	}

	posterGive ( worldModelGrid3DPosterId );
	ODEBUG2 ( " ....Reading From Poster Finished" );

	if ( planner->removeEnv( ) !=KD_OK )
	{
		ODEBUG1 ( "Error in removing environment ");
		return FAIL ;
	}
	
	if ( planner->updateEnvironmentKppPolyhedron ( outRangeMap ) ==KD_ERROR )
	{
		ODEBUG1 ( "ERROR :  updateEnvironment of the planner(KppVoxel)." );
		return FAIL ;
	}
	if ( planner->loadOccupiedEnv() !=KD_OK )
	{
		ODEBUG1 ( "Error in loadingCompleteEnv environment ");
		return FAIL ;
	}

	return ETHER;
}

/*------------------------------------------------------------------------
 * LoadCompleteEnv
 *
 * Description: 
 *
 * Reports:      OK
 *              S_worldModelPlanner_LOAD_COMPLETE_ENV_FAILURE
 */

/* worldModelPlannerLoadCompleteEnv  -  codel EXEC of LoadCompleteEnv
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerLoadCompleteEnv(int *report)
{
	std::vector<char> outRangeMap;
	if ( posterTake ( worldModelGrid3DPosterId, POSTER_READ ) != OK )
	{
		*report = errnoGet();
		return FAIL;
	}
	int inNbOfVoxel = map3DModel->nbOfGridX * map3DModel->nbOfGridY * map3DModel->nbOfGridZ;
	outRangeMap.resize ( inNbOfVoxel );
	for ( int i=0 ; i<inNbOfVoxel ;i++ )
	{
		outRangeMap[i] = map3DModel->gridvalue[i];
	}

	posterGive ( worldModelGrid3DPosterId );
	ODEBUG2 ( " ....Reading From Poster Finished" );

	if ( planner->removeEnv( ) !=KD_OK )
	{
		ODEBUG1 ( "Error in removing environment ");
		return FAIL ;
	}
	
	if ( planner->updateEnvironmentKppPolyhedron ( outRangeMap ) ==KD_ERROR )
	{
		ODEBUG1 ( "ERROR :  updateEnvironment of the planner(KppVoxel)." );
		return FAIL ;
	}
	if ( planner->loadCompleteEnv() !=KD_OK )
	{
		ODEBUG1 ( "Error in loadingCompleteEnv environment ");
		return FAIL ;
	}

	return ETHER;
}



/*------------------------------------------------------------------------
 * NextBestView
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_FAIL_IN_CALCULATING_THE_NEXT_BEXT_VIEW
 */

/* worldModelPlannerNextBextView  -  codel EXEC of NextBestView
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerNextBestView ( WalkFromToPosition *fromTo, WalkPosition *nextBestPos, int *report )
{
	if ( !SDI_F->initialized ) { *report = S_worldModelPlanner_NOT_INITIALIZED; return ETHER; }

	double fromRzRad = fromTo->fromPosition.RzPos * M_PI/180.;
	double toRzRad = fromTo->toPosition.RzPos * M_PI/180.;
	std::vector<double> nbv= planner->nextBextView ( fromTo->fromPosition.xPos, fromTo->fromPosition.yPos, fromRzRad, fromTo->toPosition.xPos, fromTo->toPosition.yPos, toRzRad );
	SDI_F->nextBestPos.xPos=nbv.at ( 0 );
	SDI_F->nextBestPos.yPos=nbv.at ( 1 );
	SDI_F->nextBestPos.RzPos=nbv.at ( 2 ) *180/M_PI;

	return ETHER;
}

/*------------------------------------------------------------------------
 * Visibility
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_FAIL_IN_CALCULATING_THE_VISIBILITY
 */

/* worldModelPlannerVisibility  -  codel EXEC of Visibility
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerVisibility ( WalkFromToPosition *fromTo,double *potentialVisibility, int *report )
{
	if ( !SDI_F->initialized ) { *report = S_worldModelPlanner_NOT_INITIALIZED; return ETHER; }

	double fromRzRad = fromTo->fromPosition.RzPos * M_PI/180.;
	double toRzRad = fromTo->toPosition.RzPos * M_PI/180.;
	SDI_F->potentialVisibility= planner->visibility (fromTo->fromPosition.xPos, fromTo->fromPosition.yPos, fromRzRad, fromTo->toPosition.xPos, fromTo->toPosition.yPos, toRzRad );
	return ETHER;
}

/*------------------------------------------------------------------------
 * GoToPosition
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_NOT_INITIALIZED
 *              S_worldModelPlanner_NO_SOLUTION
 */

/* worldModelPlannerGoToPositionMain  -  codel EXEC of GoToPosition
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerGoToPositionMain ( WalkFromToPosition *fromTo, int *successFlag, int *report )
{
	if ( !SDI_F->initialized ) { *report = S_worldModelPlanner_NOT_INITIALIZED; return ETHER; }
	SDI_F->successFlag=1;
	double fromRzRad = fromTo->fromPosition.RzPos * M_PI/180.;
	double toRzRad = fromTo->toPosition.RzPos * M_PI/180.;

	ODEBUG2 ( " initConfig : " << fromTo->fromPosition.xPos << " , " << fromTo->fromPosition.yPos << " , " << fromTo->fromPosition.RzPos << " Degree" );
	ODEBUG2 ( " goalConfig : " << fromTo->toPosition.xPos << " , " <<fromTo->toPosition.yPos << " , " << fromTo->toPosition.RzPos << " Degree" );

	planner->initConfig ( fromTo->fromPosition.xPos, fromTo->fromPosition.yPos,fromRzRad );
	planner->goalConfig ( fromTo->toPosition.xPos, fromTo->toPosition.yPos, toRzRad );

	if ( planner->solve() != KD_OK )
	{
		// *report = S_worldModelPlanner_NO_SOLUTION;
		ODEBUG2 ( "There is no solution. " );
		SDI_F->successFlag=0;
		return ETHER ;
	}
//----------------------------------------Update the poster of trajectory
	vector<CkwsConfigShPtr> path;
	vector<double> vecConfig;
	vector<double> walkTime;
	vector< vector<double> > zmp;
	vector< vector<double> > waist;
	unsigned int walkIndex, i, nbConfig;

	i=0;
	nbConfig = 0;

	STATUS s = planner->solution ( path, walkTime, zmp, waist );

	for ( walkIndex = 0; walkIndex < walkTime.size(); walkIndex=walkIndex+20 )
	{
		path[walkIndex]->getDofValues ( vecConfig );
		nbConfig++;

		for ( unsigned int j=0; j < vecConfig.size();j++ )
		{
			SDI_F->walkHrp2Traj.dof[i]=vecConfig[j];
			i++;
		}
	}
	SDI_F->walkHrp2Traj.nbDof=vecConfig.size();
	SDI_F->walkHrp2Traj.nbConfig=nbConfig;
	ODEBUG2 ( "walkHrp2Traj.nbDof = " << vecConfig.size() );
	ODEBUG2 ( "walkHrp2Traj.nbConfig = " << nbConfig );
	ODEBUG2 ( "walkTime.size() = " << walkTime.size() );
	return ETHER;
}

/*------------------------------------------------------------------------
 * ClearRoadmap
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_CLEAR_ROADMAP_FAILURE
 */

/* worldModelPlannerClearRoadmap  -  codel EXEC of ClearRoadmap
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerClearRoadmap ( int *report )
{
	if ( !SDI_F->initialized ) { *report = S_worldModelPlanner_NOT_INITIALIZED; return ETHER; }
	if ( planner->clearRoadmap() != KD_OK )
	{
		*report = S_worldModelPlanner_CLEAR_ROADMAP_FAILURE;
		ODEBUG1 ( "Error in clearing roadmap. " );

		return ETHER;
	}
	return ETHER;
}

/*------------------------------------------------------------------------
 * UpdateRoadmap
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_UPDATE_ROADMAP_FAILURE
 */

/* worldModelPlannerUpdateRoadmap  -  codel EXEC of UpdateRoadmap
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerUpdateRoadmap ( int *report )
{
	if ( !SDI_F->initialized ) { *report = S_worldModelPlanner_NOT_INITIALIZED; return ETHER; }
	if ( planner->updateRoadmap() != KD_OK )
	{
		*report = S_worldModelPlanner_UPDATE_ROADMAP_FAILURE;
		ODEBUG1 ( " Error in updating roadmap " );
		return ETHER;
	}
	return ETHER;
}

/*------------------------------------------------------------------------
 * GetConfig
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_NO_SOLUTION
 *              S_worldModelPlanner_NO_MODEL
 */

/* worldModelPlannerGetConfigMain  -  codel EXEC of GetConfig
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerGetConfigMain ( int *report )
{
	double robotConfig[40];
	ghrp2_config_t cfg;
	POSTER_ID p;
	STATUS s;
	ktStatus r;
	int i;

	s = posterFind ( "hrp2State", &p );
	if ( s != OK ) { *report = S_worldModelPlanner_NO_MODEL; return ETHER; }

	if ( posterRead ( p, 0, &cfg, sizeof ( cfg ) ) != sizeof ( cfg ) )
	{
		*report = S_worldModelPlanner_NO_MODEL; return ETHER;
	}

	for ( i=0; i<40; i++ )
	{
		robotConfig[i] = cfg.angles[i];
	}

	r = planner->initOpenHRPPosturePatGenRad ( robotConfig );
	if ( r != KD_OK )
	{
		*report = S_worldModelPlanner_NO_SOLUTION;
		return ETHER;
	}
	return ETHER;
}

/*------------------------------------------------------------------------
 * SavePath
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_NO_SUCH_FILE
 */

/* worldModelPlannerSavePathMain  -  codel EXEC of SavePath
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerSavePathMain ( PathName *name, int *report )
{
	FILE *fpos, *frpy, *fzmp;
	char full[1024];
	ktStatus s;

	vector<CkwsConfigShPtr> path;
	vector<double> walkTime;
	vector< vector<double> > zmp;
	vector< vector<double> > waist;
	vector<double> ohrp ( 40 );
	unsigned int walkIndex, i;

	/* get data */
	s = planner->solution ( path, walkTime, zmp, waist );
	if ( path.size() < 1 ) return ETHER;

	/* open pos */
	sprintf ( full, "%s.pos", name->name );
	fpos = fopen ( full, "w" );
	if ( !fpos )
	{
		*report = S_worldModelPlanner_NO_SUCH_FILE; return ETHER;
	}

	/* open rpy */
	sprintf ( full, "%s.rpy", name->name );
	frpy = fopen ( full, "w" );
	if ( !frpy )
	{
		fclose ( fpos );
		*report = S_worldModelPlanner_NO_SUCH_FILE; return ETHER;
	}

	/* open zmp */
	sprintf ( full, "%s.zmp", name->name );
	fzmp = fopen ( full, "w" );
	if ( !fzmp )
	{
		fclose ( fpos ); fclose ( frpy );
		*report = S_worldModelPlanner_NO_SUCH_FILE; return ETHER;
	}

	/* write data */
	for ( walkIndex = 0; walkIndex < walkTime.size(); walkIndex++ )
	{
		fprintf ( fpos, "%g ", walkTime[walkIndex] );
		fprintf ( frpy, "%g ", walkTime[walkIndex] );
		fprintf ( fzmp, "%g ", walkTime[walkIndex] );

		toOpenHRPConfig ( path[walkIndex], ohrp );
		for ( i = 0; i<ohrp.size(); i++ ) fprintf ( fpos, "%g ", ohrp[i] );
		fprintf ( fpos, "\n" );

		fprintf ( frpy, "%g %g %g\n",
		          waist[walkIndex][3], waist[walkIndex][4], waist[walkIndex][5] );

		fprintf ( fzmp, "%g %g %g\n",
		          zmp[walkIndex][0], zmp[walkIndex][1], zmp[walkIndex][2] );
	}

	/* done */
	fclose ( fpos ); fclose ( frpy ); fclose ( fzmp );
	return ETHER;
}


/*------------------------------------------------------------------------
 * ExecutePath
 *
 * Description:
 *
 * Reports:      OK
 *              S_worldModelPlanner_CANNOT_PUSH
 */

/* worldModelPlannerExecutePathStart  -  codel START of ExecutePath
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
static vector<CkwsConfigShPtr> path;
static vector<double> walkTime;
static vector< vector<double> > zmp;
static vector< vector<double> > waist;
static unsigned int walkIndex;

ACTIVITY_EVENT
worldModelPlannerExecutePathStart ( int *report )
{
	if ( !SDI_F->initialized ) { *report = S_worldModelPlanner_NOT_INITIALIZED; return ETHER; }

	ktStatus s;

	s = planner->solution ( path, walkTime, zmp, waist );
	walkIndex = 0;
	if ( path.size() < 1 ) return ETHER;
	return EXEC;
}

/* worldModelPlannerExecutePathMain  -  codel EXEC of ExecutePath
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
worldModelPlannerExecutePathMain ( int *report )
{
	int i, n;
	vector<double> ohrp;
	ghrp2_config_t cfg;

	ohrp.resize ( 40 );

	/* flood fill the buffer */
	for ( walkIndex = 0; walkIndex < walkTime.size(); walkIndex++ )
	{
		cfg.time = walkTime[walkIndex];

		toOpenHRPConfig ( path[walkIndex], ohrp );
		for ( i = 0; i<GHRP2_MAXDOF; i++ ) cfg.angles[i] = ohrp[i];

		cfg.zmp[0] = zmp[walkIndex][0];
		cfg.zmp[1] = zmp[walkIndex][1];
		cfg.zmp[2] = zmp[walkIndex][2];

		cfg.waistRpy[0] = waist[walkIndex][3];
		cfg.waistRpy[1] = waist[walkIndex][4];
		cfg.waistRpy[2] = waist[walkIndex][5];

		n = netshring_push ( shtarget, &cfg, sizeof ( cfg ) );
		if ( n < 0 || n != sizeof ( cfg ) )
		{
			*report = S_worldModelPlanner_CANNOT_PUSH;
			return ETHER;
		}

		if ( walkIndex % 500 == 0 ) printf ( "wrote %g\n", cfg.time );
	}

	printf ( "wrote config up to time %g\n", cfg.time );

	cfg.time = -1.0;
	n = netshring_push ( shtarget, &cfg, sizeof ( cfg ) );
	if ( n < 0 || n != sizeof ( cfg ) )
	{
		*report = S_worldModelPlanner_CANNOT_PUSH;
		return ETHER;
	}

	return ETHER;
}

